"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModuleCommands = void 0;
/**
 * Data structure describing routes and commands for a particular module (which may be the entire project),
 * including execute methods (if any)
 */
class ModuleCommands {
    constructor(routeMap = new Map(), execMethodDataSet = new Set()) {
        this.routeMap = routeMap;
        this.execMethodDataSet = execMethodDataSet;
        this.routesByCommandName = new Map();
        for (const [route, commandSet] of routeMap) {
            for (const { command } of commandSet) {
                const routes = this.routesByCommandName.get(command) ?? new Set();
                routes.add(route);
                this.routesByCommandName.set(command, routes);
            }
        }
    }
    /**
     * Returns `true` if this instance has some actual data
     */
    get hasData() {
        return Boolean(this.execMethodDataSet.size + this.routeMap.size);
    }
}
exports.ModuleCommands = ModuleCommands;
//# sourceMappingURL=module-commands.js.map